# SutekhGui.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details
"""SutekhGui.py: start the GUI"""


import gtk
from sutekh.core.SutekhObjects import VersionTable, aObjectList
from sutekh.SutekhUtility import prefs_dir, ensure_dir_exists, sqlite_uri
from sutekh.gui.MultiPaneWindow import MultiPaneWindow
from sutekh.core.DatabaseVersion import DatabaseVersion
from sqlobject import sqlhub, connectionForURI
from sutekh.gui.ConfigFile import ConfigFile
from sutekh.gui.GuiDBManagement import do_db_upgrade, initialize_db
from sutekh.gui.SutekhDialog import do_complaint_error
from sutekh.SutekhInfo import SutekhInfo
import sys, optparse, os, traceback


# Script Launching

def parse_options(aArgs):
    """SutekhGui's option parsing"""
    oOptParser = optparse.OptionParser(usage="usage: %prog [options]",
            version="%%prog %s" % SutekhInfo.VERSION_STR)
    oOptParser.add_option("-d", "--db",
                  type="string", dest="db", default=None,
                  help="Database URI. [sqlite://$PREFSDIR$/sutekh.db]")
    oOptParser.add_option("--ignore-db-version",
                  action="store_true", dest="ignore_db_version", default=False,
                  help="Ignore the database version check. Only use this if " \
                          "you know what you're doing.")
    oOptParser.add_option("--rcfile",
                  type="string", dest="sRCFile", default=None,
                  help="Specify Alternative resources file. " \
                          "[~/.sutekh/sutekhrc or " \
                          "$APPDATA$/Sutekh/sutekhrc]")
    oOptParser.add_option("--sql-debug",
                  action="store_true", dest="sql_debug", default=False,
                  help="Print out SQL statements.")
    oOptParser.add_option("--verbose",
            action="store_true", dest="verbose", default=False,
            help="Display warning messages")
    return oOptParser, oOptParser.parse_args(aArgs)

def exception_handler(oType, oValue, oTraceback):
    """sys.excepthook exception handler."""
    if oType == KeyboardInterrupt:
        # don't complain about KeyboardInterrupts
        return

    sMessage = "Sutekh reported an unhandled exception:\n" \
        "%s\n" % (str(oValue),)

    oErrorDlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR,
        gtk.BUTTONS_CLOSE, sMessage)
    oErrorDlg.set_name("Sutekh.dialog")

    aTraceback = traceback.format_exception(oType, oValue, oTraceback)
    oErrorDlg.format_secondary_text("".join(aTraceback))

    # we ignore the response here
    oErrorDlg.run()
    oErrorDlg.destroy()

def main(aArgs):
    """Start the Sutekh Gui.

       Check that database exists, doesn't need to be upgraded, then
       pass control off to MultiPaneWindow.
       Save preferences on exit if needed
       """
    # handle exceptions with a GUI dialog
    sys.excepthook = exception_handler

    oOptParser, (oOpts, aArgs) = parse_options(aArgs)
    sPrefsDir = prefs_dir("Sutekh")

    if len(aArgs) != 1:
        oOptParser.print_help()
        return 1

    if oOpts.sRCFile is None:
        ensure_dir_exists(sPrefsDir)
        oOpts.sRCFile = os.path.join(sPrefsDir, "sutekhrc")

    oConfig = ConfigFile(oOpts.sRCFile)

    if oOpts.db is None:
        oOpts.db = oConfig.get_database_uri()

    if oOpts.db is None:
        # No commandline + no rc entry
        ensure_dir_exists(sPrefsDir)
        oOpts.db = sqlite_uri(os.path.join(sPrefsDir, "sutekh.db"))
        oConfig.set_database_uri(oOpts.db)

    oConn = connectionForURI(oOpts.db)
    sqlhub.processConnection = oConn

    if oOpts.sql_debug:
        oConn.debug = True

    # Check we have the correct gtk version
    sMessage = gtk.check_version(2, 10, 0)
    if sMessage is not None:
        do_complaint_error('Incorrect gtk version. Sutekh requires at least'
                ' gtk 2.10.0.\nError reported %s' % sMessage)
        return 1

    # construct Window
    oMultiPaneWindow = MultiPaneWindow()

    # Test on some tables where we specify the table name
    if not oConn.tableExists('abstract_card') or \
            not oConn.tableExists('physical_map'):
        if not initialize_db(oMultiPaneWindow):
            return 1

    aTables = [VersionTable] + aObjectList
    aVersions = []

    for oTable in aTables:
        aVersions.append(oTable.tableversion)

    oVer = DatabaseVersion()

    if not oVer.check_tables_and_versions(aTables, aVersions) and \
            not oOpts.ignore_db_version:
        aLowerTables, aHigherTables =  oVer.get_bad_tables(aTables, aVersions)
        if not do_db_upgrade(aLowerTables, aHigherTables):
            return 1

    oMultiPaneWindow.setup(oConfig, oOpts.verbose)
    oMultiPaneWindow.run()

    # Save Config Changes
    oConfig.write()

if __name__ == "__main__":
    sys.exit(main(sys.argv))





# CardSetExportJOL.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Plugin for exporting to JOL"""

import gtk
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhFileWidget import ExportDialog
from sutekh.io.WriteJOL import WriteJOL
from sutekh.SutekhUtility import safe_filename

class CardSetExportJOL(CardListPlugin):
    """Provides a dialog for selecting a filename, then calls on
       WriteJOL to produce the required output."""
    dTableVersions = { PhysicalCardSet: [2, 3, 4, 5]}
    aModelsSupported = [PhysicalCardSet]

    def get_menu_item(self):
        """Register with the 'Export Card Set' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oExport = gtk.MenuItem("Export to JOL format")
        oExport.connect("activate", self.make_dialog)
        return ('Export Card Set', oExport)

    # pylint: disable-msg=W0613
    # oWidget has to be here, although it's unused
    def make_dialog(self, oWidget):
        """Create the dialog"""
        oDlg = ExportDialog("Choose FileName for Exported CardSet",
                self.parent, '%s.jol.txt' % safe_filename(self.view.sSetName))
        oDlg.add_filter_with_pattern('Text Files', ['*.txt'])
        oDlg.run()
        self.handle_response(oDlg.get_name())

    # pylint: enable-msg=W0613

    def handle_response(self, sFileName):
        """Handle the users response. Write the text output to file."""
        # pylint: disable-msg=E1101
        # SQLObject methods confuse pylint
        if sFileName is not None:
            oCardSet = self.get_card_set()
            if not oCardSet:
                return
            oWriter = WriteJOL()
            fOut = file(sFileName,"w")
            oWriter.write(fOut, oCardSet)
            fOut.close()

# pylint: disable-msg=C0103
# accept plugin name
plugin = CardSetExportJOL



# WhiteWolfParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# WhiteWolf Parser
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# Copyright 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""HTML Parser for extracting cards from the WW online cardlist."""

import HTMLParser, re
from logging import Logger
from sutekh.core.SutekhObjects import SutekhObjectMaker

# Card Saver

def _find_sect_and_title(aLines):
    """Search the first 2 lines of the card text for sect & title
       information.

       This is potentially brittle, since it had to rely on WW's
       standard text layout.
       """
    # pylint: disable-msg=R0912, R0915
    # R0912: Need to consider all the cases here
    # R0915: This is thus a long function
    # Card text for vampires is either Sect attributes. or Sect.
    sSect = None
    sTitle = None
    if aLines[0].find('Camarilla') != -1:
        sSect = 'Camarilla'
        if aLines[0].find('Camarilla primogen') != -1:
            sTitle = 'Primogen'
        elif aLines[0].find('Camarilla Prince of') != -1:
            sTitle = 'Prince'
        elif aLines[0].find('Justicar') != -1:
            # Since Justicar titles are of the form
            # 'Camarilla <Clan> Justicar'
            oJusticar = re.compile('Camarilla [A-Z][a-z]* Justicar')
            if oJusticar.search(aLines[0]) is not None:
                sTitle = 'Justicar'
        # Inner circle my be either Camariila Inner Circle or
        # Camarilla Clan Inner Circle
        # Hopefully this will go away sometime
        elif aLines[0].find('Inner Circle') != -1:
            if aLines[0].find('Camarilla Inner Circle') != -1:
                sTitle = 'Inner Circle'
            else:
                oInnerCircle = re.compile('Camarilla [A-Z][a-z]* Inner Circle')
                if oInnerCircle.search(aLines[0]) is not None:
                    sTitle = 'Inner Circle'
    elif aLines[0].find('Sabbat') != -1:
        sSect = 'Sabbat'
        if aLines[0].find('Sabbat Archbishop of') != -1:
            sTitle = 'Archbishop'
        elif aLines[0].find('Sabbat bishop') != -1:
            sTitle = 'Bishop'
        elif aLines[0].find('Sabbat priscus') != -1:
            sTitle = 'Priscus'
        elif aLines[0].find('Sabbat cardinal') != -1:
            sTitle = 'Cardinal'
        elif aLines[0].find('Sabbat regent') != -1:
            sTitle = 'Regent'
    elif aLines[0].find('Independent') != -1:
        sSect = 'Independent'
        # Independent titles are on the next line. Of the form
        # 'Name has X vote(s)'
        # pylint: disable-msg=W0704
        # error isn't fatal, so ignoring it is fine
        try:
            # Special cases 'The Baron' and 'Ur-Shulgi' mean we don't
            # anchor the regexp
            oIndTitle = re.compile('[A-Z][a-z]* has ([0-9]) vote')
            oMatch = oIndTitle.search(aLines[1])
            if oMatch is not None and not aLines[1].startswith('[MERGED]'):
                oMergedTitle = re.compile('MERGED.*has ([0-9]) vote')
                oMergedMatch = oMergedTitle.search(aLines[1])
                if oMergedMatch is not None and oMergedMatch.groups()[0] == \
                        oMatch.groups()[0]:
                    pass
                elif oMatch.groups()[0] == '1':
                    sTitle = 'Independent with 1 vote'
                elif oMatch.groups()[0] == '2':
                    sTitle = 'Independent with 2 votes'
                elif oMatch.groups()[0] == '3':
                    sTitle = 'Independent with 3 votes'
        except IndexError:
            pass
    elif aLines[0].find('Laibon') != -1:
        sSect = 'Laibon'
        if aLines[0].find('Laibon magaji') != -1:
            sTitle = 'Magaji'
    return sSect, sTitle

class CardDict(dict):
    """Dictionary object which holds the extracted card info."""
    oDisGaps = re.compile(r'[\\\/{}\&\s]+')
    oWhiteSp = re.compile(r'[{}\s]+')
    oDispCard = re.compile(r'\[[^\]]+\]$')

    def __init__(self, oLogger):
        super(CardDict, self).__init__()
        self.oLogger = oLogger
        self._oMaker = SutekhObjectMaker()

    def _parse_text(self):
        """Parse the CardText for Sect and Titles"""
        sType = None
        if self.has_key('cardtype'):
            sTypes = self['cardtype']
            # Determine if vampire is one of the card types
            for sVal in sTypes.split('/'):
                if sVal == 'Vampire':
                    sType = sVal
        # Check for REFLEX card type
        if self['text'].find(' [REFLEX] ') != -1:
            if self.has_key('cardtype'):
                # append to card types
                self['cardtype'] += '/Reflex'
            else:
                self['cardtype'] = 'Reflex'
        if sType == 'Vampire':
            # Sect attributes: more text. Title is in the attributes
            aLines = self['text'].split(':')
            sSect, sTitle = _find_sect_and_title(aLines)
            # check if the vampire has flight (text ends has Flight [FLIGHT].)
            oFlightRexegp = re.compile('Flight \[FLIGHT\]\.')
            oMatch = oFlightRexegp.search(aLines[-1])
            if oMatch:
                if self.has_key('discipline'):
                    self['discipline'] += ' FLI'
                else:
                    self['discipline'] = 'FLI'
            if sSect is not None:
                self['sect'] = sSect
            if sTitle is not None:
                self['title'] = sTitle

    def _add_blood_shadowed_court(self, oCard):
        """Add Blood Shadowed Court to the expansion list if appropriate."""
        oCamVampPair = self._oMaker.make_rarity_pair('CE', 'Vampire')
        if oCamVampPair in oCard.rarity:
            oPair = self._oMaker.make_rarity_pair('BSC', 'BSC')
            if oPair not in oCard.rarity:
                # Don't duplicate entries
                oCard.addRarityPair(oPair)

    def _make_card(self, sName):
        """Create the abstract card in the database."""
        sName = self.oDispCard.sub('', sName)
        return self._oMaker.make_abstract_card(sName)

    def _add_expansions(self, oCard, sExp):
        """Add expansion information to the card, creating expansion pairs
           as needed."""
        aPairs = [x.split(':') for x in sExp.strip('[]').split(',')]
        aExp = []
        for aPair in aPairs:
            if len(aPair) == 1:
                aExp.append((aPair[0].strip(), 'NA'))
            elif aPair[1].strip().startswith('Promo-'):
                # Handle the TR:Promo special case
                aExp.append((aPair[1].strip(), 'NA'))
            else:
                aExp.append((aPair[0].strip(), aPair[1].strip()))

        for sExp, sRarSet in aExp:
            for sRar in sRarSet.split('/'):
                oPair = self._oMaker.make_rarity_pair(sExp, sRar)
                oCard.addRarityPair(oPair)

    def _add_disciplines(self, oCard, sDis):
        """Add the list of disciplines to the card, creating discipline
           pairs as needed."""
        sDis = self.oDisGaps.sub(' ', sDis).strip()

        if sDis == '-none-' or sDis == '':
            return

        for sVal in sDis.split():
            if  sVal == sVal.lower():
                oPair = self._oMaker.make_discipline_pair(sVal, 'inferior')
            else:
                oPair = self._oMaker.make_discipline_pair(sVal, 'superior')
            oCard.addDisciplinePair(oPair)

    def _add_virtues(self, oCard, sVir):
        """Add the list of virtues to the card."""
        sVir = self.oDisGaps.sub(' ', sVir).strip()

        if sVir == '-none-' or sVir == '':
            return

        for sVal in sVir.split():
            oVirt = self._oMaker.make_virtue(sVal)
            oCard.addVirtue(oVirt)

    def _add_creeds(self, oCard, sCreed):
        """Add creeds to the card."""
        sCreed = self.oWhiteSp.sub(' ', sCreed).strip()

        if sCreed == '-none-' or sCreed == '':
            return

        for sVal in sCreed.split('/'):
            oCard.addCreed(self._oMaker.make_creed(sVal.strip()))

    def _add_clans(self, oCard, sClan):
        """Add clans to the card."""
        sClan = self.oWhiteSp.sub(' ', sClan).strip()

        if sClan == '-none-' or sClan == '':
            return

        for sVal in sClan.split('/'):
            oCard.addClan(self._oMaker.make_clan(sVal.strip()))

    def _add_cost(self, oCard, sCost):
        """Add the cost to the card, replace 'X' with -1."""
        sCost = self.oWhiteSp.sub(' ', sCost).strip()
        sAmnt, sType = sCost.split()

        if sAmnt.lower() == 'x':
            iCost = -1
        else:
            iCost = int(sAmnt, 10)

        oCard.cost = iCost
        oCard.costtype = str(sType.lower()) # make str non-unicode

    def _add_group(self, oCard, sGroup):
        """Add the group to the card. Replace '*' with -1."""
        sGroup = self.oWhiteSp.sub(' ', sGroup).strip()

        if sGroup == '*':
            iGroup = -1
        else:
            iGroup = int(sGroup, 10)

        oCard.group = iGroup

    def _add_life(self, oCard, sLife):
        """Add the life to the card."""
        sLife = self.oWhiteSp.sub(' ', sLife).strip()
        aLife = sLife.split()
        # pylint: disable-msg=W0704
        # ignoring the error is the right thing here
        try:
            oCard.life = int(aLife[0], 10)
        except ValueError:
            pass

    def _get_level(self, sLevel):
        """Normalised the level string."""
        return self.oWhiteSp.sub(' ', sLevel).strip().lower()

    def _add_level(self, oCard, sLevel):
        """Add the correct string for the level to the card."""
        oCard.level = str(self._get_level(sLevel)) # make str non-unicode

    def _add_level_to_name(self, sName, sLevel):
        """Add level info to the vampire name."""
        return sName.strip() + " (%s)" % self._get_level(sLevel).capitalize()

    def _add_capacity(self, oCard, sCap):
        """Add the capacity to the card."""
        sCap = self.oWhiteSp.sub(' ', sCap).strip()
        aCap = sCap.split()
        # pylint: disable-msg=W0704
        # ignoring the error is the right thing here
        try:
            oCard.capacity = int(aCap[0], 10)
        except ValueError:
            pass

    def _add_card_type(self, oCard, sTypes):
        """Add the card type info to the card."""
        for sVal in sTypes.split('/'):
            oCard.addCardType(self._oMaker.make_card_type(sVal.strip()))

    def _add_title(self, oCard, sTitle):
        """Add the title to the card."""
        oCard.addTitle(self._oMaker.make_title(sTitle))

    def _add_sect(self, oCard, sSect):
        """Add the sect to the card."""
        oCard.addSect(self._oMaker.make_sect(sSect))

    def _add_physical_cards(self, oCard):
        """Create a physical card for each expansion."""
        self._oMaker.make_physical_card(oCard, None)
        for oExp in set([oRarity.expansion for oRarity in oCard.rarity]):
            self._oMaker.make_physical_card(oCard, oExp)

    def save(self):
        # pylint: disable-msg=R0912
        # Need to consider all cases, so many branches
        """Commit the card to the database.

           This fills in the needed fields and creates entries in the join
           tables as needed.
           """
        if not self.has_key('name'):
            return

        if self.has_key('text'):
            self._parse_text()

        if self.has_key('level'):
            self['name'] = self._add_level_to_name(self['name'], self['level'])

        self.oLogger.info('Card: %s', self['name'])

        oCard = self._make_card(self['name'])
        if self.has_key('group'):
            self._add_group(oCard, self['group'])

        if self.has_key('capacity'):
            self._add_capacity(oCard, self['capacity'])

        if self.has_key('cost'):
            self._add_cost(oCard, self['cost'])

        if self.has_key('life'):
            self._add_life(oCard, self['life'])

        if self.has_key('level'):
            self._add_level(oCard, self['level'])

        if self.has_key('expansion'):
            self._add_expansions(oCard, self['expansion'])

        if self.has_key('discipline'):
            self._add_disciplines(oCard, self['discipline'])

        if self.has_key('virtue'):
            self._add_virtues(oCard, self['virtue'])

        if self.has_key('clan'):
            self._add_clans(oCard, self['clan'])

        if self.has_key('creed'):
            self._add_creeds(oCard, self['creed'])

        if self.has_key('cardtype'):
            self._add_card_type(oCard, self['cardtype'])

        if self.has_key('burn option'):
            oCard.burnoption = True

        if self.has_key('title'):
            self._add_title(oCard, self['title'])

        if self.has_key('sect'):
            self._add_sect(oCard, self['sect'])

        if self.has_key('text'):
            oCard.text = self['text'].replace('\r', '')

        self._add_blood_shadowed_court(oCard)

        self._add_physical_cards(oCard)

        oCard.syncUpdate()

# State Base Classes

class StateError(Exception):
    """Error case in the state true"""
    pass

class State(object):
    """Base class for the State transitions."""
    def __init__(self, oLogger):
        super(State, self).__init__()
        self._sData = ""
        self.oLogger = oLogger

    def transition(self, sTag, dAttr):
        """Transition from one state to another"""
        raise NotImplementedError

    def data(self, sData):
        """Add data to the state"""
        self._sData += sData

class StateWithCard(State):
    """Base class for state transitions when already in a card."""
    # pylint: disable-msg=W0223
    # descendants will override transition, so still abstract here.
    def __init__(self, dInfo, oLogger):
        super(StateWithCard, self).__init__(oLogger)
        self._dInfo = dInfo

# State Classes

class NoCard(State):
    """State when we are not in a card."""
    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transition to PotentialCard if needed."""
        if sTag == 'p':
            return PotentialCard(self.oLogger)
        else:
            return self

class PotentialCard(State):
    """State for a section that may be a card"""
    def transition(self, sTag, dAttr):
        """Transition to InCard or NoCard if needed."""
        if sTag == 'a' and dAttr.has_key('name'):
            return InCard(CardDict(self.oLogger), self.oLogger)
        else:
            return NoCard(self.oLogger)

class InCard(StateWithCard):
    """State for in a card description in the WW card list."""
    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transistion to the appropriate section state if needed."""
        if sTag == 'p':
            raise StateError()
        elif sTag == '/p':
            self._dInfo.save()
            return NoCard(self.oLogger)
        elif sTag == 'span' and dAttr.get('class') == 'cardname':
            return InCardName(self._dInfo, self.oLogger)
        elif sTag == 'span' and dAttr.get('class') == 'exp':
            return InExpansion(self._dInfo, self.oLogger)
        elif sTag == 'span' and dAttr.get('class') == 'key':
            return InKeyValue(self._dInfo, self.oLogger)
        elif sTag == 'td' and dAttr.get('colspan') == '2':
            return InCardText(self._dInfo, self.oLogger)
        else:
            return self

class InCardName(StateWithCard):
    """In the card name section."""
    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transistion back to InCard if needed."""
        if sTag == '/span':
            self._dInfo['name'] = self._sData.strip()
            return InCard(self._dInfo, self.oLogger)
        elif sTag == 'span':
            raise StateError()
        else:
            return self

class InExpansion(StateWithCard):
    """In the expansions section."""
    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transistion back to InCard if needed."""
        if sTag == '/span':
            self._dInfo['expansion'] = self._sData.strip()
            return InCard(self._dInfo, self.oLogger)
        elif sTag == 'span':
            raise StateError()
        else:
            return self

class InCardText(StateWithCard):
    """In the card text section."""
    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transistion back to InCard if needed."""
        if sTag == '/td' or sTag == 'tr' or sTag == '/tr' or sTag == '/table':
            self._dInfo['text'] = self._sData.strip()
            return InCard(self._dInfo, self.oLogger)
        elif sTag == 'td':
            raise StateError()
        else:
            return self

class InKeyValue(StateWithCard):
    """Extract a dictionary key from the table holding the card info."""
    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transition to WaitingForValue if needed."""
        if sTag == '/span':
            sKey = self._sData.strip().strip(':').lower()
            return WaitingForValue(sKey, self._dInfo, self.oLogger)
        elif sTag == 'span':
            raise StateError()
        else:
            return self

class WaitingForValue(StateWithCard):
    """Extract a value from the table holding the card info."""
    def __init__(self, sKey, dInfo, oLogHandler):
        super(WaitingForValue, self).__init__(dInfo, oLogHandler)
        self._sKey = sKey
        self._bGotTd = False

    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transition back to InCard if needed."""
        if sTag == 'td':
            self._sData = ""
            self._bGotTd = True
            return self
        elif sTag == '/td' and self._bGotTd:
            self._dInfo[self._sKey] = self._sData.strip()
            return InCard(self._dInfo, self.oLogger)
        elif sTag == '/tr':
            self._dInfo[self._sKey] = None
            return InCard(self._dInfo, self.oLogger)
        elif sTag == 'tr':
            raise StateError()
        else:
            return self

# Parser

class WhiteWolfParser(HTMLParser.HTMLParser, object):
    """Actual Parser for the WW cardlist HTML file(s)."""
    def __init__(self, oLogHandler):
        # super().__init__ calls reset, so we need this first
        self.oLogger = Logger('White wolf card parser')
        if oLogHandler is not None:
            self.oLogger.addHandler(oLogHandler)
        super(WhiteWolfParser, self).__init__()
        self._oState = NoCard(self.oLogger)

    # pylint: disable-msg=C0111
    # names are as listed in HTMLParser docs, so no need for docstrings
    def reset(self):
        super(WhiteWolfParser, self).reset()
        self._oState = NoCard(self.oLogger)

    def handle_starttag(self, sTag, aAttr):
        self._oState = self._oState.transition(sTag.lower(), dict(aAttr))

    def handle_endtag(self, sTag):
        self._oState = self._oState.transition('/'+sTag.lower(), {})

    def handle_data(self, sData):
        self._oState.data(sData)

    # pylint: disable-msg=C0321
    # these don't need statements
    def handle_charref(self, sName): pass
    def handle_entityref(self, sName): pass





# test_ARDBTextParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Test case for ARDB text parser"""

from sutekh.io.ARDBTextParser import ARDBTextParser
from sutekh.tests.TestCore import DummyHolder
import unittest

class ARDBTextParserTests(unittest.TestCase):
    """class for the ARDB/FELDB text input parser"""
    sTestText1 = """
        Deck Name : Test Deck
        Author : Anon Y Mous
        Description :
        Simple test deck.

        http://www.example.url/in/description

        Crypt [3 vampires] Capacity min: 2 max: 10 average: 7.33
       ------------------------------------------------------------
        2x Test Vamp 1			  10 ...
        1x Test Vamp 2			  2  ...
        ...

        Library [19 cards]
        ------------------------------------------------------------
        Action [6]
            2x Test Card 1
            4x Test Card 2

        Action Modifier [12]
            12x Test Card 3

        Reaction [1]
            1x Test Card 4
        ...
    """

    sTestText2 = """
        Deck Name : Test Deck 2
        Author : Anon Y Mous
        Description : Simple test deck.
        Crypt: (3 vampires, Min: 2, Max: 10 Ave: 7.33)
       ------------------------------------------------------------
        2 Test Vamp 1			  aus dom for   10 ...
        1 Test Vamp 2			  DOM for obf   2  ...
        ...

        Library [19 cards]
        ------------------------------------------------------------
        Action [6]
        2 Test Card 1
        4 Test Card 2

        Action Modifier [12]
        12 Test Card 3

        Reaction [1]
        1 Test Card 4
        ...
    """


    def test_basic(self):
        """Run the input test."""
        oHolder = DummyHolder()
        oParser = ARDBTextParser(oHolder)

        for sLine in self.sTestText1.split("\n"):
            oParser.feed(sLine + "\n")

        self.assertEqual(oHolder.name, "Test Deck")
        self.assertEqual(oHolder.author, "Anon Y Mous")
        self.failUnless(oHolder.comment.startswith(
            "        Simple test deck."))
        self.failUnless(oHolder.comment.endswith("in/description\n\n"))

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 6)
        self.failUnless(("Test Vamp 1", 2) in aCards)
        self.failUnless(("Test Vamp 2", 1) in aCards)
        self.failUnless(("Test Card 1", 2) in aCards)
        self.failUnless(("Test Card 2", 4) in aCards)
        self.failUnless(("Test Card 3", 12) in aCards)
        self.failUnless(("Test Card 4", 1) in aCards)

        oHolder = DummyHolder()
        oParser = ARDBTextParser(oHolder)
        for sLine in self.sTestText2.split("\n"):
            oParser.feed(sLine + "\n")

        self.assertEqual(oHolder.name, "Test Deck 2")
        self.assertEqual(oHolder.author, "Anon Y Mous")
        self.failUnless(oHolder.comment.startswith(
            "Simple test deck."))

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 6)
        self.failUnless(("Test Vamp 1", 2) in aCards)
        self.failUnless(("Test Vamp 2", 1) in aCards)
        self.failUnless(("Test Card 1", 2) in aCards)
        self.failUnless(("Test Card 2", 4) in aCards)
        self.failUnless(("Test Card 3", 12) in aCards)
        self.failUnless(("Test Card 4", 1) in aCards)


if __name__ == "__main__":
    unittest.main()



# ChangeGroupBy.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Allow the use to change how the cards are grouped in the CardListView"""

import gtk
from sutekh.core.SutekhObjects import PhysicalCard, PhysicalCardSet
from sutekh.core.Groupings import CardTypeGrouping, ClanGrouping, \
        DisciplineGrouping, ExpansionGrouping, RarityGrouping, \
        CryptLibraryGrouping, NullGrouping, MultiTypeGrouping, \
        SectGrouping, TitleGrouping, CostGrouping
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhDialog import SutekhDialog

class GroupCardList(CardListPlugin):
    """Plugin to allow the user to change how cards are grouped.

       Show a dialog which allows the user to select from the avail
       groupings of the cards, and changes the setting in the CardListView.
       """
    dTableVersions = {}
    aModelsSupported = [PhysicalCard, PhysicalCardSet]

    # pylint: disable-msg=W0142
    # ** magic OK here
    def __init__(self, *aArgs, **kwargs):
        super(GroupCardList, self).__init__(*aArgs, **kwargs)
        self._dGrpings = {}
        self._dGrpings['Card Type'] = CardTypeGrouping
        self._dGrpings['Multi Card Type'] = MultiTypeGrouping
        self._dGrpings['Crypt or Library'] = CryptLibraryGrouping
        self._dGrpings['Clans and Creeds'] = ClanGrouping
        self._dGrpings['Disciplines and Virtues'] = DisciplineGrouping
        self._dGrpings['Expansion'] = ExpansionGrouping
        self._dGrpings['Rarity'] = RarityGrouping
        self._dGrpings['Sect'] = SectGrouping
        self._dGrpings['Title'] = TitleGrouping
        self._dGrpings['Cost'] = CostGrouping
        self._dGrpings['No Grouping'] = NullGrouping
        self._oFirstBut = None # placeholder for the radio group

    def get_menu_item(self):
        """Register on the 'Plugins' menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oGrouping = gtk.MenuItem("Change Grouping")
        oGrouping.connect("activate", self.activate)
        return ('Plugins', oGrouping)

    # pylint: disable-msg=W0613
    # oWidget required by function signature
    def activate(self, oWidget):
        """Response to the menu - create the dialog and run it"""
        oDlg = self.make_dialog()
        oDlg.run()

    # pylint: enable-msg=W0613

    def make_dialog(self):
        """Create the required dialog."""
        sName = "Change Card List Grouping..."

        oDlg = SutekhDialog(sName, self.parent,
                gtk.DIALOG_DESTROY_WITH_PARENT)
        oDlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        oDlg.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)

        oDlg.connect("response", self.handle_response)

        cCurrentGrping = self.get_grouping()
        oIter = self._dGrpings.iteritems()
        # pylint: disable-msg=E1101
        # vbox confuses pylint
        for sName, cGrping in oIter:
            self._oFirstBut = gtk.RadioButton(None, sName, False)
            self._oFirstBut.set_active(cGrping is cCurrentGrping)
            oDlg.vbox.pack_start(self._oFirstBut)
            break
        for sName, cGrping in oIter:
            oBut = gtk.RadioButton(self._oFirstBut, sName)
            oBut.set_active(cGrping is cCurrentGrping)
            oDlg.vbox.pack_start(oBut)

        oDlg.show_all()

        return oDlg

    # Actions

    def handle_response(self, oDlg, oResponse):
        """Handle the response from the dialog.

           Change the grouping in the CardListView if appropriate"""
        if oResponse == gtk.RESPONSE_CANCEL:
            oDlg.destroy()
        elif oResponse == gtk.RESPONSE_OK:
            for oBut in self._oFirstBut.get_group():
                if oBut.get_active():
                    sLabel = oBut.get_label()
                    cGrping = self._dGrpings[sLabel]
                    self.set_grouping(cGrping)
            oDlg.destroy()

    def set_grouping(self, cGrping):
        """Set the grouping to that specified by cGrping."""
        self.model.groupby = cGrping
        # Use view.load so we get busy cursor, etc.
        self.view.load()

    def get_grouping(self):
        """Get the current grouping class."""
        return self.model.groupby

# pylint: disable-msg=C0103
# accept plugin name
plugin = GroupCardList



# WriteArdbXML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Based off the Anarach Revolt Deck Builder xml support,
# ARDB (c) Francios Gombalt, Christoph Boget, Ville Virta and Vincent Ripoll
# GPL - see COPYING for details

"""Given a list of Abstract Cards in a set, write a XML file compatable with
   the Anarch Revolt Deck Builder."""

from sutekh.core.SutekhObjects import IAbstractCard
from sutekh.core.ArdbInfo import ArdbInfo, escape_ardb_name
from sutekh.SutekhUtility import pretty_xml
import time
# pylint: disable-msg=E0611, F0401
# xml.etree is a python2.5 thing
try:
    from xml.etree.ElementTree import Element, SubElement, ElementTree, \
            tostring
except ImportError:
    from elementtree.ElementTree import Element, SubElement, ElementTree, \
            tostring
# pylint: enable-msg=E0611, F0401


class WriteArdbXML(ArdbInfo):
    """Reformat cardset to elementTree and export it to a ARDB
       compatible XML file."""

    def gen_tree(self, sSetName, sAuthor, sDescription, dCards):
        """Creates the actual XML document into memory."""
        # pylint: disable-msg=R0914
        # Need this many local variables to create proper XML tree
        oRoot = Element('deck')

        sDateWritten = time.strftime('%Y-%m-%d', time.localtime())
        oRoot.attrib['generator'] = "Sutekh [ %s ]" % self.sVersionString
        oRoot.attrib['formatVersion'] = self.sFormatVersion
        oRoot.attrib['databaseVersion'] = self.sDatabaseVersion
        oNameElem = SubElement(oRoot, 'name')
        oNameElem.text  = sSetName
        oAuthElem = SubElement(oRoot, 'author')
        oAuthElem.text = sAuthor
        oDescElem = SubElement(oRoot, 'description')
        oDescElem.text = sDescription
        oDateElem = SubElement(oRoot, 'date')
        oDateElem.text = sDateWritten

        dVamps, dCryptStats = self._extract_crypt(dCards)
        (dLib, iLibSize) = self._extract_library(dCards)

        oCryptElem = SubElement(oRoot, 'crypt', size=str(dCryptStats['size']),
                min=str(dCryptStats['min']), max=str(dCryptStats['max']),
                avg='%.2f' % dCryptStats['avg'])
        self.format_vamps(oCryptElem, dVamps)

        oLibElem = SubElement(oRoot, 'library', size=str(iLibSize))
        self.format_library(oLibElem, dLib)

        pretty_xml(oRoot)
        return oRoot

    def format_vamps(self, oCryptElem, dVamps):
        """Convert the Vampire dictionary into elementtree representation."""
        # pylint: disable-msg=R0914
        # Need this many local variables to create proper XML tree
        for tKey in sorted(dVamps, key = lambda x: x[1]):
            iNum = dVamps[tKey]
            iId, sName, sSet = tKey
            # pylint: disable-msg=E1101
            # IAbstractCard confuses pylint
            oCard = IAbstractCard(sName)
            oCardElem = SubElement(oCryptElem, 'vampire',
                    databaseID=str(iId), count=str(iNum))
            # This won't match the ARDB ID's, unless by chance.
            # It looks like that should not be an issue as ARDB will
            # use the name if the IDs don't match
            # It's unclear to me what values ARDB uses here, but
            # these are fine for the xml2html conversion, and look meaningful
            oAdvElem = SubElement(oCardElem, 'adv')
            oNameElem = SubElement(oCardElem, 'name')
            sName = escape_ardb_name(sName)
            if oCard.level is not None:
                oAdvElem.text = '(Advanced)'
                # This is a bit hackish
                oNameElem.text = sName.replace(' (Advanced)', '')
            else:
                oNameElem.text = sName
            oSetElem = SubElement(oCardElem, 'set')
            oSetElem.text = sSet
            oDiscElem = SubElement(oCardElem, 'disciplines')
            sDisciplines = self._gen_disciplines(oCard)
            oDiscElem.text = sDisciplines
            aClan = [x.name for x in oCard.clan]
            oClanElem = SubElement(oCardElem, 'clan')
            oCapElem = SubElement(oCardElem, 'capacity')
            if len(oCard.creed) > 0:
                # ARDB seems to treat all Imbued as being of the same clan
                # Should we do an Imbued:Creed thing?
                oClanElem.text = 'Imbued'
                oCapElem.text = str(oCard.life)
            else:
                oClanElem.text = aClan[0]
                oCapElem.text = str(oCard.capacity)
            oGrpElem = SubElement(oCardElem, 'group')
            oGrpElem.text = str(oCard.group)
            # ARDB doesn't handle sect specifically
            # No idea how ARDB represents independant titles -
            # this seems set when the ARDB database is created, and is
            # not in the ARDB codebase
            if len(oCard.title) > 0:
                oTitleElem = SubElement(oCardElem, 'title')
                aTitles = [oC.name for oC in oCard.title]
                oTitleElem.text = aTitles[0]
            oTextElem = SubElement(oCardElem, 'text')
            oTextElem.text = oCard.text

    def format_library(self, oLibElem, dLib):
        """Format the dictionary of library cards for the element tree."""
        # pylint: disable-msg=R0914
        # Need this many local variables to create proper XML tree
        for tKey in sorted(dLib, key = lambda x: x[1]):
            iNum = dLib[tKey]
            iId, sName, sTypeString, sSet = tKey
            # pylint: disable-msg=E1101
            # IAbstractCard confuses pylint
            oCard = IAbstractCard(sName)
            sName = escape_ardb_name(sName)
            oCardElem = SubElement(oLibElem, 'card', databaseID=str(iId),
                    count=str(iNum))
            oNameElem = SubElement(oCardElem, 'name')
            oNameElem.text = sName
            oSetElem = SubElement(oCardElem, 'set')
            oSetElem.text = sSet
            if oCard.costtype is not None:
                oCostElem = SubElement(oCardElem, 'cost')
                oCostElem.text = "%d %s " % (oCard.cost, oCard.costtype )
            if len(oCard.clan) > 0:
                # ARDB also strores things like "requires a prince"
                # we don't so too bad
                oReqElem = SubElement(oCardElem, 'requirement')
                aClan = [x.name for x in oCard.clan]
                oReqElem.text = "/".join(aClan)
            oTypeElem = SubElement(oCardElem, 'type')
            oTypeElem.text = sTypeString
            # Not sure if this does quite the right thing here
            sDisciplines = self._gen_disciplines(oCard)
            if sDisciplines != '':
                oDiscElem = SubElement(oCardElem, 'disciplines')
                oDiscElem.text = sDisciplines
            oTextElem = SubElement(oCardElem, 'text')
            oTextElem.text = oCard.text

    # pylint: disable-msg=R0913
    # we need all these arguments
    def write(self, fOut, sSetName, sAuthor, sDescription, oCardIter):
        """Takes filename, deck details and a dictionary of cards, of the
           form dCard[(id,name)] = count and writes the file."""
        dCards = self._get_cards(oCardIter)
        oRoot = self.gen_tree(sSetName, sAuthor, sDescription, dCards)
        ElementTree(oRoot).write(fOut)

    # pylint: enable-msg=R0913

    def gen_xml_string(self, sSetName, sAuthor, sDescription, oCardIter):
        """Generate string XML representation"""
        dCards = self._get_cards(oCardIter)
        oRoot = self.gen_tree(sSetName, sAuthor, sDescription, dCards)
        return tostring(oRoot)





# TestCore.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Base for Sutekh test cases"""

from sutekh.tests.TestData import TEST_CARD_LIST, TEST_RULINGS
from sutekh.SutekhUtility import read_white_wolf_list, read_rulings, \
        refresh_tables
from sutekh.core.SutekhObjects import aObjectList
from sutekh.io.WwFile import WwFile
from sqlobject import sqlhub, connectionForURI
import unittest
import tempfile
import os
from logging import FileHandler

class SutekhTest(unittest.TestCase):
    """Base class for Sutekh tests.

       Define common setup and teardown routines common to test cases.
       """

    def _create_tmp_file(self, sData=None):
        """Creates a temporary file with the given data, closes it and returns
           the filename."""
        (fTemp, sFilename) = tempfile.mkstemp(dir=self._sTempDir)
        os.close(fTemp)
        self._aTempFiles.append(sFilename)

        if sData:
            fTmp = file(sFilename, "wb")
            fTmp.write(sData)
            fTmp.close()

        return sFilename


    # pylint: disable-msg=C0103
    # setUp + tearDown names are needed by unittest - use their convention
    # pylint: disable-msg=W0201
    # setUp is always called by the tests, so it doesn't matter that
    # declarations aren't in __init__
    def _setUpTemps(self):
        """Create a directory to hold the temporary files."""
        self._sTempDir = tempfile.mkdtemp(suffix='dir', prefix='sutekhtests')
        self._aTempFiles = []

    def _tearDownTemps(self):
        """Clean up the temporary files."""
        for sFile in self._aTempFiles:
            if os.path.exists(sFile):
                # Tests may clean up their own temp files
                os.remove(sFile)
        os.rmdir(self._sTempDir)
        self._sTempDir = None
        self._aTempFiles = None

    def _setUpDb(self):
        """Initialises a database with the cardlist and
           rulings.
           """
        oConn = sqlhub.processConnection

        sCardList = self._create_tmp_file(TEST_CARD_LIST)
        sRulings = self._create_tmp_file(TEST_RULINGS)

        assert refresh_tables(aObjectList, oConn)

        oLogHandler = FileHandler('/dev/null')
        read_white_wolf_list([WwFile(sCardList)], oLogHandler)
        read_rulings(WwFile(sRulings), oLogHandler)

    def setUp(self):
        """Common setup routine for tests.

           Initialises a database with the cardlist and rulings.
           """
        self._setUpTemps()
        # Get the database to use from the environment, defaulting to an
        # sqlite memory DB
        sDBUrl = os.getenv('SUTEKH_TEST_DB', "sqlite:///:memory:")
        oConn = connectionForURI(sDBUrl)
        sqlhub.processConnection = oConn
        self._setUpDb()

    def tearDown(self):
        """Common teardown routine for tests.

           Base sqlite cleanup.
           """
        # Clean up database
        aObjectList.reverse()
        for cCls in aObjectList:
            cCls.dropTable(ifExists=True)
        aObjectList.reverse()
        sqlhub.processConnection = None
        self._tearDownTemps()


class DummyHolder(object):
    """Emulate CardSetHolder for test purposes."""
    def __init__(self):
        self.dCards = {}
        # pylint: disable-msg=C0103
        # placeholder names for CardSetHolder attributes
        self.name = ''
        self.comment = ''
        self.author = ''

    def add(self, iCnt, sName, sExpName):
        """Add a card to the dummy holder."""
        self.dCards.setdefault((sName, sExpName), 0)
        self.dCards[(sName, sExpName)] += iCnt

    def get_cards_exps(self):
        """Get the cards with expansions"""
        return self.dCards.items()

    def get_cards(self):
        """Get the card info without expansions"""
        dNoExpCards = {}
        for sCardName, sExpName in self.dCards:
            dNoExpCards.setdefault(sCardName, 0)
            dNoExpCards[sCardName] += self.dCards[(sCardName, sExpName)]
        return dNoExpCards.items()



# test_WriteELDBDeckFile.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an ELDB deck"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import aCardSetNames, \
        get_phys_cards
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.io.WriteELDBDeckFile import WriteELDBDeckFile
import unittest

sExpected = """"Test Set 1"
"A test author"
"A test comment"
1
"Abebe"
4
".44 Magnum"
"AK-47"
"Abbot"
"Abombwe"
"""

class ELDBDeckWriterTests(SutekhTest):
    """class for the ELDB deck writer tests"""

    def test_deck_writer(self):
        """Test ELDB deck writing"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        aAddedPhysCards = get_phys_cards()
        # We have a physical card list, so create some physical card sets
        oPhysCardSet1 = PhysicalCardSet(name=aCardSetNames[0])
        oPhysCardSet1.comment = 'A test comment'
        oPhysCardSet1.author = 'A test author'

        for iLoop in range(5):
            oPhysCardSet1.addPhysicalCard(aAddedPhysCards[iLoop].id)
            oPhysCardSet1.syncUpdate()

        # Check output

        oWriter = WriteELDBDeckFile()
        sTempFileName =  self._create_tmp_file()
        fOut = open(sTempFileName, 'w')
        oWriter.write(fOut, oPhysCardSet1)
        fOut.close()

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sData, sExpected)

if __name__ == "__main__":
    unittest.main()



# CountCardSetCards.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>,
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Display a running total of the cards in a card set"""

import gtk
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.CardListModel import CardListModelListener

def _id_card(oCard):
    """Idenitfy the card type"""
    sType = list(oCard.cardtype)[0].name
    if sType == 'Vampire' or sType == 'Imbued':
        return 'Crypt'
    else:
        return 'Library'



class CountCardSetCards(CardListPlugin, CardListModelListener):
    """Listen to changes on the card list views, and display a toolbar
       containing a label with a running count of the cards in the card
       set, the library cards and the crypt cards
       """
    dTableVersions = {PhysicalCardSet : [5]}
    aModelsSupported = [PhysicalCardSet]

    sFormat = 'Tot: <b>%(tot)d</b> L: <b>%(lib)d</b> C: <b>%(crypt)d</b>'
    sTooltip = 'Total Cards: <b>%(tot)d</b> (Library: <b>%(lib)d</b>' \
            ' Crypt: <b>%(crypt)d</b>)'

    # pylint: disable-msg=W0142
    # **magic OK here
    def __init__(self, *aArgs, **kwargs):
        super(CountCardSetCards, self).__init__(*aArgs, **kwargs)

        self.__iTot = 0
        self.__iCrypt = 0
        self.__iLibrary = 0
        self.__oTextLabel = None

        # We only add listeners to windows we're going to display the toolbar
        # on
        if self.check_versions() and self.check_model_type():
            self.model.add_listener(self)
    # pylint: enable-msg=W0142

    def get_toolbar_widget(self):
        """Overrides method from base class."""
        if not self.check_versions() or not self.check_model_type():
            return None

        self.__oTextLabel = gtk.Label(self.sFormat % {'tot' : 0,
            'crypt' : 0, 'lib' : 0})
        if hasattr(self.__oTextLabel, 'set_tooltip_markup'):
            self.__oTextLabel.set_tooltip_markup(self.sTooltip % {
            'tot' : 0, 'crypt' : 0, 'lib' : 0})


        return self.__oTextLabel

    def update_numbers(self):
        """Update the label"""
        # Timing issues mean that this can be called before text label has
        # been properly realised, so we need this guard case
        if self.__oTextLabel:
            self.__oTextLabel.set_markup(self.sFormat % {'tot' : self.__iTot,
                'crypt' : self.__iCrypt, 'lib' : self.__iLibrary})
            if hasattr(self.__oTextLabel, 'set_tooltip_markup'):
                self.__oTextLabel.set_tooltip_markup(self.sTooltip % {
                'tot' : self.__iTot, 'crypt' : self.__iCrypt,
                'lib' : self.__iLibrary})

    def load(self, aAbsCards):
        """Listen on load events & update counts"""
        self.__iCrypt = 0
        self.__iLibrary = 0
        self.__iTot = len(aAbsCards)
        for oAbsCard in aAbsCards:
            if _id_card(oAbsCard) == 'Crypt':
                self.__iCrypt += 1
            else:
                self.__iLibrary += 1
        self.update_numbers()

    def alter_card_count(self, oCard, iChg):
        """respond to alter_card_count events"""
        self.__iTot += iChg
        if _id_card(oCard) == 'Crypt':
            self.__iCrypt += iChg
        else:
            self.__iLibrary += iChg
        self.update_numbers()

    def add_new_card(self, oCard):
        """response to add_new_card events"""
        self.__iTot += 1
        if _id_card(oCard) == 'Crypt':
            self.__iCrypt += 1
        else:
            self.__iLibrary += 1
        self.update_numbers()

# pylint: disable-msg=C0103
# accept plugin name
plugin = CountCardSetCards



# SearchDialog.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Special search dialog for the TreeViews for displaying the card list.

   This is designed to avoid issues with the default TreeView search on
   windows with gtk 2.12"""

import gtk
import sys

class SearchDialog(gtk.Window):
    # pylint: disable-msg=R0904
    # gtk Widget, so many public methods
    """Mini dialog that contains a gtk.Entry item for handling card
       searches"""

    def __init__(self, oView, oWin):
        super(SearchDialog, self).__init__()
        self._oView = oView
        self._oWin = oWin
        self.set_transient_for(oWin)
        # We want the WM to ignore this window - this looks the best choice
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_MENU)
        self.set_modal(True)
        self.set_decorated(False)
        oFrame = gtk.Frame()
        self.oEntry = gtk.Entry(30)
        self.oEntry.set_has_frame(True)
        self.oEntry.set_inner_border(None)
        self.set_flags(gtk.CAN_FOCUS)

        oAlign = gtk.Alignment()
        oAlign.set_padding(3, 3, 3, 3)
        oAlign.add(self.oEntry)
        oFrame.add(oAlign)
        self.add(oFrame)
        self.connect('key-press-event', self._key_press)
        self.oEntry.connect('activate', lambda x: self.hide())
        # Work around wierd windows behaviour where the button-press-event
        # doesn't get delivered to us
        if sys.platform.startswith("win"):
            self.connect('button-release-event', self.check_hide)
        else:
            self.connect('button-press-event', self.check_hide)
        self.set_focus_on_map(True)
        # Hook up to the View
        oView.set_search_entry(self.oEntry)
        oView.set_enable_search(False)
        # Key combination for searching
        oView.connect_after('key-press-event',
                self.treeview_key_press_event)

    def set_text(self, sString):
        """Set the contents of the entry to sString"""
        self.oEntry.set_text(sString)
        self.oEntry.set_position(1)

    def hide(self):
        """Hide the window"""
        super(SearchDialog, self).hide()

    # pylint: disable-msg=W0613
    # Various arguments required by function signatures
    def _key_press(self, oWidget, oEvent):
        """Hide the dialog on escape"""
        if oEvent.keyval == gtk.gdk.keyval_from_name('Escape'):
            self.hide()
            return True
        return False # propogate event

    def check_hide(self, oWidget, oEvent):
        """Check if we need to hide the dialog"""
        # pylint: disable-msg=E1101
        # allocation confuses pylint
        (iXPos, iYPos) = self.get_pointer()
        oGeom = self.allocation
        if (iXPos < 0) or (iYPos < 0) or (iXPos > oGeom.width) or \
                (iYPos > oGeom.height):
            # Button event outside the dialog, so hide
            self.hide()

    def show_all(self):
        """Move the window into position when shown"""
        oWin = self._oView.get_bin_window()
        tWinPos = oWin.get_origin()
        tViewSize = oWin.get_size()
        # We first move totally off-screen, then show so we
        # can get our size
        # This is basically the same approach as the default gtk search
        # dialog takes, so we more-or-less duplicate it's behaviour
        oScreen = self._oView.get_screen()
        self.move(oScreen.get_width() + tViewSize[0],
                oScreen.get_height() +  tViewSize[1])
        super(SearchDialog, self).show_all()
        oThisWinAlloc = self.get_allocation()
        iXPos = tWinPos[0] + tViewSize[0] - oThisWinAlloc.width
        iYPos = tWinPos[1] + tViewSize[1]
        # Special case limits as gtk does
        if iXPos < 0:
            iXPos = 0
        elif iXPos + oThisWinAlloc.width > oScreen.get_width():
            iXPos = oScreen.get_width() - oThisWinAlloc.width
        if iYPos < 0:
            iYPos = 0
        elif iYPos + oThisWinAlloc.height > oScreen.get_height():
            iYPos = oScreen.get_height() - oThisWinAlloc.height
        self.move(iXPos, iYPos)
        self._oWin.set_focus(self.oEntry)
        self.oEntry.grab_focus()
        # Force the widget to be focus - probablity unsure
        self.oEntry.set_flags(gtk.HAS_FOCUS)

    # key press for searching in the tree view

    def treeview_key_press_event(self, oWidget, oEvent):
        """Handle key press events, so we display a search box with
           entry completion."""
        if oEvent.keyval in range(33, 127):
            self.show_all()
            if len(oEvent.string) > 0:
                # Add text to search dialog
                self.set_text(oEvent.string)
        # connect_after, so everything else has had it's shot
        return True




# CardSetExportArdbXML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Plugin for exporting to ARDB's XML format"""

import gtk
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhFileWidget import ExportDialog
from sutekh.io.WriteArdbXML import WriteArdbXML
from sutekh.io.WriteArdbInvXML import WriteArdbInvXML
from sutekh.SutekhUtility import safe_filename

class CardSetExportArdbXML(CardListPlugin):
    """Provides a dialog for selecting a filename, then calls on
       WriteArdbXML to produce the required output."""
    dTableVersions = { PhysicalCardSet: [2, 3, 4, 5]}
    aModelsSupported = [PhysicalCardSet]

    def get_menu_item(self):
        """Register on the 'Export Card Set' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oExport = gtk.MenuItem("Export to ARDB XML")
        oExport.connect("activate", self.make_dialog)
        return ('Export Card Set', oExport)

    # pylint: disable-msg=W0613
    # oWidget has to be here, although it's unused
    def make_dialog(self, oWidget):
        """Create the dialog"""
        # pylint: disable-msg=E1101
        # vbox confuses pylint
        oDlg = ExportDialog("Choose FileName for Exported CardSet",
                self.parent, '%s_ARDB.xml' % safe_filename(self.view.sSetName))
        oDlg.add_filter_with_pattern('XML Files', ['*.xml'])
        oFirstBut = gtk.RadioButton(None, 'ARDB Deck XML File', False)
        oDlg.vbox.pack_start(oFirstBut, expand=False)
        oSecondBut = gtk.RadioButton(oFirstBut, 'ARDB Inventory XML File')
        oDlg.vbox.pack_start(oSecondBut, expand=False)
        oDlg.show_all()
        oDlg.run()

        self.handle_response(oDlg.get_name(), oFirstBut)

    # pylint: enable-msg=W0613

    def handle_response(self, sFileName, oFirstBut):
        """Handle the users response. Write the XML output to file."""
        # pylint: disable-msg=E1101
        # SQLObject methods confuse pylint
        if sFileName is not None:
            oCardSet = self.get_card_set()
            if not oCardSet:
                return
            fOut = file(sFileName,"w")
            if oFirstBut.get_active():
                oWriter = WriteArdbXML()
                oWriter.write(fOut, self.view.sSetName, oCardSet.author,
                        oCardSet.comment, self.get_all_cards())
            else:
                oWriter = WriteArdbInvXML()
                oWriter.write(fOut, self.get_all_cards())
            fOut.close()

# pylint: disable-msg=C0103
# accept plugin name
plugin = CardSetExportArdbXML



# RulingParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# WhiteWolf Rulings Parser
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""HTML Parser for extracting card rulings from the WW online rulings list."""

import HTMLParser, re
from logging import Logger
from sutekh.core.SutekhObjects import IAbstractCard, SutekhObjectMaker
from sqlobject import SQLObjectNotFound

# Ruling Saver

class RuleDict(dict):
    """Dictionary object which holds the extracted rulings information."""
    _aSectionKeys = ['title', 'card']
    _oMasterOut = re.compile(r'\s*-\s*Master\s*\:?\s*Out\-of\-Turn$')
    _oCommaThe = re.compile(r'\s*\,\s*The$')
    _dOddTitles = {
        'Absilmilard\'s Army' : 'Absimiliard\'s Army',
        'Carlotta' : 'Carlotta Giovanni',
        'Donal O\'Connor' : u'D\xf3nal O\'Connor',
        'Herald of Topeth' : 'Herald of Topheth',
        'Illusions of Kindred' : 'Illusions of the Kindred',
        'Khobar Towers' : 'Khobar Towers, Al-Khubar',
        'Mehemet of the Ahl-i-Batin' : 'Mehemet of the Ahl-i-Batin (Mage)',
        'Lazar Dobrescu' : u'L\xe1z\xe1r Dobrescu',
        'Merill Molitor' : 'Merrill Molitor',
        'Rotschreck' : u'R\xf6tschreck',
        'Seattle Committe' : 'Seattle Committee',
        'Shackles of Enkindu' : 'Shackles of Enkidu',
        'Shadow Court Satyr' : 'Shadow Court Satyr (Changeling)',
        'Smiling Jack the Anarch' : 'Smiling Jack, The Anarch',
        'Tereza Rotas' : 'Tereza Rostas',
        'Ur-Shulgi' : 'Ur-Shulgi, The Shepherd',
    }

    def __init__(self, oLogger):
        self.oLogger = oLogger
        super(RuleDict, self).__init__()
        self._oMaker = SutekhObjectMaker()

    def _find_card(self, sTitle):
        """Find the abstract card this rules applies to."""
        sTitle = self._oMasterOut.sub('', sTitle)
        sTitle = self._oCommaThe.sub('', sTitle)

        # pylint: disable-msg=W0704
        # Skipping SQLObject exceptions is the right thing to do here
        try:
            return IAbstractCard(sTitle)
        except SQLObjectNotFound:
            pass

        try:
            return IAbstractCard(self._dOddTitles[sTitle])
        except KeyError:
            pass
        except SQLObjectNotFound:
            pass

        try:
            return IAbstractCard(('The ' + sTitle))
        except SQLObjectNotFound:
            pass

        return None

    def clear_rule(self):
        """Remove current contents of the rule."""
        dKeep = {}
        for sKey in self._aSectionKeys:
            if self.has_key(sKey):
                dKeep[sKey] = self[sKey]
        self.clear()
        for (sKey, sValue) in dKeep.items():
            self[sKey] = sValue

    def save(self):
        """Add the rulings to the database."""
        if not (self.has_key('title') and self.has_key('code') \
                and self.has_key('text')):
            return

        if not self.has_key('card'):
            self['card'] = self._find_card(self['title'])

        if self['card'] is None:
            return

        self.oLogger.info('Card: %s', self['card'].name)

        oRuling = self._oMaker.make_ruling(self['text'], self['code'])

        if self.has_key('url'):
            oRuling.url = self['url']

        self['card'].addRuling(oRuling)

# State Base Classes

class StateError(Exception):
    """Error thrown in state transitions."""
    pass

class State(object):
    """Base class for all States"""
    def __init__(self, oLogger):
        super(State, self).__init__()
        self._sData = ""
        self.oLogger = oLogger

    def transition(self, sTag, dAttr):
        """Transition from one state to another"""
        raise NotImplementedError

    def data(self, sData):
        """Add data to the state"""
        self._sData += sData

class StateWithRule(State):
    """Base class for states which contain information of interest"""
    # pylint: disable-msg=W0223
    # transition method is still abstract here
    def __init__(self, dInfo, oLogger):
        super(StateWithRule, self).__init__(oLogger)
        self._dInfo = dInfo

# State Classes

class NoSection(State):
    """Not in any ruling section."""
    # pylint: disable-msg=W0613
    # dAttr needed by function signature
    def transition(self, sTag, dAttr):
        """Transition ot InSection if needed."""
        if sTag == 'p':
            return InSection(RuleDict(self.oLogger), self.oLogger)
        else:
            return self

class InSection(StateWithRule):
    """In a ruling section."""
    # pylint: disable-msg=W0613
    # dAttr needed by function signature
    def transition(self, sTag, dAttr):
        """Transition to SectionTitle or NoSection as needed."""
        if sTag == 'b':
            return SectionTitle(self._dInfo, self.oLogger)
        elif sTag == 'p':
            # skip to next section
            return InSection(RuleDict(self.oLogger), self.oLogger)
        else:
            return NoSection(self.oLogger)

class SectionTitle(StateWithRule):
    """In the tilte of the section."""
    # pylint: disable-msg=W0613
    # dAttr needed by function signature
    def transition(self, sTag, dAttr):
        """Transition to SectionWithTitle if needed."""
        if sTag == 'b':
            raise StateError()
        elif sTag == '/b':
            self._dInfo['title'] = self._sData.strip().strip(':')
            return SectionWithTitle(self._dInfo, self.oLogger)
        else:
            return self

class SectionWithTitle(StateWithRule):
    """In a section with a known title."""
    # pylint: disable-msg=W0613
    # dAttr needed by function signature
    def transition(self, sTag, dAttr):
        """Transition to SectionRule, InSection or NoSection if needed."""
        if sTag == 'li':
            return SectionRule(self._dInfo, self.oLogger)
        elif sTag == 'p':
            # skip to next section
            return InSection(RuleDict(self.oLogger), self.oLogger)
        elif sTag == '/p':
            return NoSection(self.oLogger)
        else:
            return self

class SectionRule(StateWithRule):
    """In a ruling in the section."""
    def transition(self, sTag, dAttr):
        """Transition to the appropriate InRule State."""
        if sTag == 'span' and dAttr.get('class') in ['ruling', 'errata',
                'clarification']:
            return InRuleText(self._dInfo, self.oLogger)
        elif sTag == 'a':
            self._dInfo['url'] = dAttr['href']
            return InRuleUrl(self._dInfo, self.oLogger)
        elif sTag == '/li':
            if not self._dInfo.has_key('code'):
                self._dInfo['code'] = self._sData.strip()
            self._dInfo.save()
            self._dInfo.clear_rule()
            return SectionWithTitle(self._dInfo, self.oLogger)
        elif sTag == 'li':
            # handles unclosed <li> inside section block
            # skip to next rule
            if not self._dInfo.has_key('code'):
                self._dInfo['code'] = self._sData.strip()
            self._dInfo.save()
            self._dInfo.clear_rule()
            return SectionRule(self._dInfo, self.oLogger)
        elif sTag == '/p':
            # handles unclosed <li> at end of section block
            # skip to next section
            if not self._dInfo.has_key('code'):
                self._dInfo['code'] = self._sData.strip()
            self._dInfo.save()
            self._dInfo.clear_rule()
            return NoSection(self.oLogger)
        return self

class InRuleText(StateWithRule):
    """In the text of a ruling."""
    # pylint: disable-msg=W0613
    # dAttr needed by function signature
    def transition(self, sTag, dAttr):
        """Transition to SectionRule if needed."""
        if sTag == 'span':
            raise StateError()
        elif sTag == '/span':
            self._dInfo['text'] = self._sData.strip()
            return SectionRule(self._dInfo, self.oLogger)
        else:
            return self

class InRuleUrl(StateWithRule):
    """In the url associated with this ruling."""
    # pylint: disable-msg=W0613
    # dAttr needed by function signature
    def transition(self, sTag, dAttr):
        """Transition to SectionRule if needed."""
        if sTag == 'a':
            raise StateError()
        elif sTag == '/a':
            self._dInfo['code'] = self._sData.strip()
            return SectionRule(self._dInfo, self.oLogger)
        else:
            return self

# Parser

class RulingParser(HTMLParser.HTMLParser, object):
    """Actual Parser for the WW rulings HTML files."""
    def __init__(self, oLogHandler):
        # super().__init__ calls reset, so we need this first
        self.oLogger = Logger('WW Rulings parser')
        if oLogHandler is not None:
            self.oLogger.addHandler(oLogHandler)
        super(RulingParser, self).__init__()
        self._oState = NoSection(self.oLogger)

    # pylint: disable-msg=C0111
    # names are as listed in HTMLParser docs, so no need for docstrings
    def reset(self):
        super(RulingParser, self).reset()
        self._oState = NoSection(self.oLogger)

    def handle_starttag(self, sTag, aAttr):
        self._oState = self._oState.transition(sTag.lower(), dict(aAttr))

    def handle_endtag(self, sTag):
        self._oState = self._oState.transition('/'+sTag.lower(), {})

    def handle_data(self, sData):
        self._oState.data(sData)

    # pylint: disable-msg=C0321
    # these don't need statements
    def handle_charref(self, sName): pass
    def handle_entityref(self, sName): pass



# test_PhysicalCardParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test PhysicalCard handling"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.io import test_WhiteWolfParser
from sutekh.core.SutekhObjects import IAbstractCard, IPhysicalCard, \
        IPhysicalCardSet, PhysicalCardSet
from sutekh.io.PhysicalCardParser import PhysicalCardParser
from sutekh.io.IdentifyXMLFile import IdentifyXMLFile
from sutekh.io.XmlFileHandling import PhysicalCardXmlFile
import unittest, os

class PhysicalCardTests(SutekhTest):
    """class for the PhysicalCard tests"""
    aExpectedCards = test_WhiteWolfParser.WhiteWolfParserTests.aExpectedCards

    def test_physical(self):
        """Test physical card handling"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables

        aAbstractCards = ['.44 magnum', 'ak-47', 'abbot', 'abebe', 'abombwe']
        # test IO

        oAC = IAbstractCard(aAbstractCards[0])
        oPC = IPhysicalCard((oAC, None))
        sLastWriterVersion = "1.0"

        sExample = '<cards sutekh_xml_version="%s"><card count="1" ' \
                   'expansion="None Specified" id="%d" name="%s" /></cards>' \
                   % (sLastWriterVersion, oPC.id, oAC.name)

        oParser = PhysicalCardParser()
        oParser.parse_string(sExample)

        oMyCollection = IPhysicalCardSet("My Collection")
        assert(len(oMyCollection.cards) == 1)
        PhysicalCardSet.delete(oMyCollection.id)

        sTempFileName =  self._create_tmp_file()
        fOut = open(sTempFileName, 'w')
        fOut.write(sExample)
        fOut.close()

        oIdFile = IdentifyXMLFile()
        oIdFile.id_file(sTempFileName)
        self.assertEqual(oIdFile.type, 'PhysicalCard')

        fIn = open(sTempFileName, 'rU')
        oParser.parse(fIn)
        fIn.close()

        oMyCollection = IPhysicalCardSet("My Collection")
        assert(len(oMyCollection.cards) == 1)
        PhysicalCardSet.delete(oMyCollection.id)

        oFile = PhysicalCardXmlFile(sTempFileName)
        oFile.read()
        oMyCollection = IPhysicalCardSet("My Collection")
        assert(len(oMyCollection.cards) == 1)
        oFile.delete()
        self.assertFalse(os.path.exists(sTempFileName))
        self.assertRaises(RuntimeError, oFile.write)


if __name__ == "__main__":
    unittest.main()



# CardSetImporter.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>,
# Copyright 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Convert a ELDB or ARDB text or html file into an Card Set."""

import gtk
import urllib2
from sutekh.io.ELDBHTMLParser import ELDBHTMLParser
from sutekh.io.ARDBTextParser import ARDBTextParser
from sutekh.io.ARDBXMLDeckParser import ARDBXMLDeckParser
from sutekh.io.ARDBXMLInvParser import ARDBXMLInvParser
from sutekh.io.ELDBDeckFileParser import ELDBDeckFileParser
from sutekh.io.ELDBInventoryParser import ELDBInventoryParser
from sutekh.io.JOLDeckParser import JOLDeckParser
from sutekh.io.LackeyDeckParser import LackeyDeckParser
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.core.CardSetHolder import CardSetHolder
from sutekh.core.CardLookup import LookupFailed
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
from sutekh.gui.SutekhFileWidget import SutekhFileWidget

class ACSImporter(CardListPlugin):
    """Convert a HTML or text deck into an ACS.

       Handles the most common formats, and allows the user to choose
       uri's, so decks published online can be easily imported.
       """
    dTableVersions = { PhysicalCardSet: [5]}
    aModelsSupported = ["MainWindow"]

    # pylint: disable-msg=W0142
    # ** magic OK
    def __init__(self, *aArgs, **kwargs):
        super(ACSImporter, self).__init__(*aArgs, **kwargs)
        # Parser classes should be instantiable using Parser(oCardSetHolder)
        # Parser objects should have a .feed(sLine) method
        self._dParsers = {
            'ELDB HTML File': ELDBHTMLParser,
            'ARDB Text File': ARDBTextParser,
            'ELDB Deck (.eld)': ELDBDeckFileParser,
            'ELDB Inventory': ELDBInventoryParser,
            'ARDB Deck XML File': ARDBXMLDeckParser,
            'ARDB Inventory XML File': ARDBXMLInvParser,
            'JOL Deck File': JOLDeckParser,
            'Lackey CCG Deck File': LackeyDeckParser,
        }
        self.oUri = None
        self.oDlg = None
        self._oFirstBut = None
        self.oFileChooser = None
        self._sNewName = ''

    def get_menu_item(self):
        """Register with the 'Import' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oImport = gtk.MenuItem("Import ARDB or ELDB Card Set")
        oImport.connect("activate", self.make_dialog)
        return ('Import Card Set', oImport)

    # pylint: disable-msg=W0613
    # oWidget required by signature
    def make_dialog(self, oWidget):
        """Create the dialog asking the user for the source to import."""
        self.oDlg = SutekhDialog("Choose Card Set File or URL", None,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK,
                 gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        # pylint: disable-msg=E1101
        # vbox confuses pylint
        self.oDlg.vbox.pack_start(gtk.Label("URL:"), expand=False)

        self.oUri = gtk.Entry(150)
        self.oUri.connect("activate", self.handle_response, gtk.RESPONSE_OK)
        self.oDlg.vbox.pack_start(self.oUri, expand=False)

        self.oDlg.vbox.pack_start(gtk.Label("OR"), expand=False)

        self.oFileChooser = SutekhFileWidget(self.parent,
                gtk.FILE_CHOOSER_ACTION_OPEN)
        self.oFileChooser.add_filter_with_pattern('HTML files', ['*.html',
            '*.htm'])
        self.oFileChooser.add_filter_with_pattern('TXT files', ['*.txt'])
        self.oFileChooser.add_filter_with_pattern('ELD files', ['*.eld'])
        self.oFileChooser.add_filter_with_pattern('XML files', ['*.xml'])
        self.oFileChooser.add_filter_with_pattern('CSV files', ['*.csv'])
        self.oFileChooser.default_filter()
        self.oDlg.vbox.pack_start(self.oFileChooser)

        # FIXME: Add some sort of 'guess format' option for the default,
        # rather than the first one in the _dParsers dict.
        oIter = self._dParsers.iterkeys()
        for sName in oIter:
            self._oFirstBut = gtk.RadioButton(None, sName, False)
            self._oFirstBut.set_active(True)
            self.oDlg.vbox.pack_start(self._oFirstBut, expand=False)
            break
        for sName in oIter:
            oBut = gtk.RadioButton(self._oFirstBut, sName)
            self.oDlg.vbox.pack_start(oBut, expand=False)

        self.oDlg.connect("response", self.handle_response)
        self.oDlg.set_size_request(600, 500)
        self.oDlg.show_all()

        self.oDlg.run()

    def handle_response(self, oWidget, oResponse):
        """Handle the user's clicking on OK or CANCEL in the dialog."""
        if oResponse == gtk.RESPONSE_OK:
            sUri = self.oUri.get_text().strip()
            sFile = self.oFileChooser.get_filename()

            for oBut in self._oFirstBut.get_group():
                sName = oBut.get_label()
                if oBut.get_active():
                    cParser = self._dParsers[sName]

            if sUri:
                self.make_cs_from_uri(sUri, cParser)
            elif sFile:
                self.make_cs_from_file(sFile, cParser)

        self.oDlg.destroy()

    def handle_name_response(self, oWidget, oResponse, oDlg, oEntry):
        """Handle the user's clicking on OK or CANCEL in the dialog."""
        if oResponse == gtk.RESPONSE_OK:
            self._sNewName = oEntry.get_text().strip()

    # pylint: enable-msg=W0613

    def make_cs_from_uri(self, sUri, cParser):
        """From an URI, create an Card Set"""
        fIn = urllib2.urlopen(sUri)
        try:
            self.make_cs(fIn, cParser)
        finally:
            fIn.close()

    def make_cs_from_file(self, sFile, cParser):
        """From an fiel, create an Card Set"""
        fIn = file(sFile, "rb")
        try:
            self.make_cs(fIn, cParser)
        finally:
            fIn.close()

    def make_cs(self, fIn, cParser):
        """Create a Abstract card set from the given file object."""
        oHolder = CardSetHolder()

        # pylint: disable-msg=W0703
        # we really do want all the exceptions
        try:
            oParser = cParser(oHolder)
            for sLine in fIn:
                oParser.feed(sLine)
        except Exception, oExp:
            sMsg = "Reading the card set failed with the following error:\n" \
                   "%s\n The file is probably not in the format the Parser" \
                   " expects.\nAborting" % oExp
            do_complaint_error(sMsg)
            # Fail out
            return

        if oHolder.num_entries < 1:
            # No cards seen, so abort
            do_complaint_error("No cards found in the card set.\n"
                    "The file may not be in the format the Parser expects.\n"
                    "Aborting")
            return

        # Check CS Doesn't Exist
        bContinue = False
        # We loop until we have an acceptable name
        while not bContinue:
            bDoNewName = False
            sMsg = ""
            if PhysicalCardSet.selectBy(name=oHolder.name).count() != 0:
                sMsg = "Card Set %s already exists.\n" \
                        "Please choose another name.\n" \
                        "Choose cancel to abort this import." % oHolder.name
                bDoNewName = True
            elif not oHolder.name:
                sMsg = "No name given for the card set\n" \
                        "Please specify a name.\n" \
                        "Choose cancel to abort this import."
                bDoNewName = True
            if bDoNewName:
                if not self.make_name_dialog(sMsg):
                    return
                oHolder.name = self._sNewName
                self._sNewName = ""
            else:
                bContinue = True

        # Create CS
        try:
            oHolder.create_pcs(oCardLookup=self.cardlookup)
        except RuntimeError, oExp:
            sMsg = "Creating the card set failed with the following error:\n"
            sMsg += str(oExp) + "\n"
            sMsg += "The file is probably not in the format the Parser" \
                    " expects\n"
            sMsg += "Aborting"
            do_complaint_error(sMsg)
            return
        except LookupFailed, oExp:
            return

        self.open_cs(oHolder.name)

    def make_name_dialog(self, sMsg):
        """Create a dialog to prompt for a new name"""
        oDlg = SutekhDialog("Choose New Card Set Name", self.parent,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK,
                    gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        oLabel = gtk.Label(sMsg)

        oEntry = gtk.Entry(50)
        # Need this so entry box works as expected
        oEntry.connect("activate", self.handle_name_response,
                gtk.RESPONSE_OK, oDlg, oEntry)
        oDlg.connect("response", self.handle_name_response, oDlg, oEntry)

        # pylint: disable-msg=E1101
        # pylint misses vbox methods
        oDlg.vbox.pack_start(oLabel)
        oDlg.vbox.pack_start(oEntry)
        oDlg.show_all()

        iResponse = oDlg.run()
        oDlg.destroy()
        return iResponse == gtk.RESPONSE_OK


# pylint: disable-msg=C0103
# accept plugin name
plugin = ACSImporter



# CardSetManagementView.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""gtk.TreeView class the card set list."""

import gtk
import unicodedata
from sutekh.gui.CardSetManagementModel import CardSetManagementModel
from sutekh.gui.SearchDialog import SearchDialog

class CardSetManagementView(gtk.TreeView, object):
    """Tree View for the card set list."""
    # pylint: disable-msg=R0904
    # gtk.Widget, so lots of public methods
    def __init__(self, oMainWindow, oController):
        self._oModel = CardSetManagementModel(oMainWindow)
        super(CardSetManagementView, self).__init__(self._oModel)

        self._oMainWin = oMainWindow
        self._oController = oController

        # Selecting rows
        self._oSelection = self.get_selection()
        self._oSelection.set_mode(gtk.SELECTION_SINGLE)

        # Text searching of card names
        self.set_search_equal_func(self.compare, None)
        # Search dialog
        # Entry item for text searching
        self._oSearchDialog = SearchDialog(self, self._oMainWin)

        # Drag and Drop
        aTargets = [ ('STRING', 0, 0),      # second 0 means TARGET_STRING
                     ('text/plain', 0, 0) ] # and here

        self.drag_source_set(gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
                aTargets, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)

        self.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                aTargets, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)

        self.connect('drag_data_get', self.drag_card_set)
        self.connect('row_activated', self._oController.row_clicked)
        self.connect('drag_data_received', self._oController.card_set_drop)

        self.set_name('card set view')
        # Grid Lines
        if hasattr(self, 'set_grid_lines'):
            self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)

        self.oNameCell = gtk.CellRendererText()
        oColumn = gtk.TreeViewColumn("Card Sets", self.oNameCell, markup=0)
        oColumn.set_expand(True)
        oColumn.set_sort_column_id(0)
        self.append_column(oColumn)
        self._oModel.load()

    # Help functions used by reload_keep_expanded
    # pylint: disable-msg=W0613
    # Various arguments required by function signatures
    def get_row_status(self, oModel, oPath, oIter, dExpandedDict):
        """Create a dictionary of rows and their expanded status."""
        if self.row_expanded(oPath):
            dExpandedDict.setdefault(oPath,
                    self._oModel.get_name_from_path(oPath))
        return False # Need to process the whole list

    # pylint: disable-msg=W0613

    def set_row_status(self, dExpandedDict):
        """Attempt to expand the rows listed in dExpandedDict."""
        for oPath in dExpandedDict:
            # pylint: disable-msg=W0704
            # Paths may disappear, so this error can be ignored
            try:
                sCardSetName = self._oModel.get_name_from_path(oPath)
                if sCardSetName == dExpandedDict[oPath]:
                    self.expand_to_path(oPath)
            except ValueError:
                pass
        return False

    # Introspection

    # pylint: disable-msg=W0212
    # We allow access via these properties (for plugins)
    mainwindow = property(fget=lambda self: self._oMainWin,
            doc="The parent window sed for dialogs, etc.")
    searchdialog = property(fget=lambda self: self._oSearchDialog,
            doc="The search dialog.")
    # pylint: enable-msg=W0212

    # Card Set Name searching

    @staticmethod
    def to_ascii(sName):
        """Convert a card name or key to a canonical ASCII form."""
        return unicodedata.normalize('NFKD', sName).encode('ascii','ignore')

    # pylint: disable-msg=R0913, W0613
    # arguments as required by the function signature
    def compare(self, oModel, iColumn, sKey, oIter, oData):
        """Compare the entered text to the card names."""

        def check_children(oModel, oIter, sKey, iLenKey):
            """Check if the children of this iterator match."""
            oChildIter = oModel.iter_children(oIter)
            while oChildIter:
                sChildName = oModel.get_name_from_iter(oChildIter)
                sChildName = sChildName[:iLenKey].lower()
                if self.to_ascii(sChildName).startswith(sKey) or\
                    sChildName.startswith(sKey):
                    oPath = oModel.get_path(oChildIter)
                    # Expand the row
                    self.expand_to_path(oPath)
                if oModel.iter_n_children(oChildIter) > 0:
                    # recurse into the children
                    check_children(oModel, oChildIter, sKey, iLenKey)
                oChildIter = oModel.iter_next(oChildIter)

        sKey = sKey.lower()
        iLenKey = len(sKey)

        # Test this row straight up
        sCardSetName = self._oModel.get_name_from_iter(oIter)[:iLenKey].lower()
        if self.to_ascii(sCardSetName).startswith(sKey) or \
                sCardSetName.startswith(sKey):
            return False

        if oModel.iter_n_children(oIter) > 0:
            # row has children, so need to check if any of the children match
            check_children(oModel, oIter, sKey, iLenKey)
        # Fell through
        return True

    def drag_card_set(self, oBtn, oDragContext, oSelectionData, oInfo, oTime):
        """Allow card sets to be dragged to a frame."""
        sSetName = self.get_selected_card_set()
        if not sSetName:
            return
        sData = "\n".join(['Card Set:', sSetName])
        oSelectionData.set(oSelectionData.target, 8, sData)

    # pylint: enable-msg=R0913, W0613

    def get_selected_card_set(self):
        """Return the currently selected card set name, or None if nothing
           is selected."""
        oModel, aSelectedRows = self._oSelection.get_selected_rows()
        if len(aSelectedRows) < 1:
            return None
        # We only allow single selection mode, so len(aSelectedRows) == 1
        oPath = aSelectedRows[0]
        return oModel.get_name_from_path(oPath)

    def get_path_at_pointer(self):
        """Get the path at the current pointer position"""
        # pylint: disable-msg=W0612
        # we ignore oIgnore
        iXPos, iYPos, oIgnore = self.get_bin_window().get_pointer()
        tRes = self.get_path_at_pos(iXPos, iYPos)
        if tRes:
            return tRes[0]
        return None



# test_PhysicalCardSetWriter.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing Card Set handling"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import aCardSetNames, \
        get_phys_cards
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.io.PhysicalCardSetWriter import PhysicalCardSetWriter
import unittest

sExpected1 = '<physicalcardset author="A test author" ' \
        'name="Test Set 1" '\
        'sutekh_xml_version="1.3">\n' \
        '  <comment>A test comment</comment>\n' \
        '  <annotations />\n' \
        '  <card count="1" expansion="None Specified" id="1"' \
        ' name=".44 Magnum" />\n' \
        '  <card count="1" expansion="None Specified" id="2"' \
        ' name="AK-47" />\n' \
        '  <card count="1" expansion="None Specified" id="8"' \
        ' name="Abbot" />\n' \
        '  <card count="1" expansion="None Specified" id="11"' \
        ' name="Abebe" />\n' \
        '  <card count="1" expansion="None Specified" id="14"' \
        ' name="Abombwe" />\n' \
        '</physicalcardset>'

sExpected2 = '<physicalcardset author="A test author" ' \
        'name="Test Set 2" '\
        'sutekh_xml_version="1.3">\n' \
        '  <comment>A formatted test comment\n' \
        'A second line\n' \
        'A third line</comment>\n' \
        '  <annotations>Some Annotations</annotations>\n' \
        '  <card count="1" expansion="Jyhad" id="1" name=".44 Magnum" />\n' \
        '  <card count="1" expansion="Lords of the Night"' \
        ' id="2" name="AK-47" />\n' \
        '  <card count="1" expansion="Third Edition" id="8"' \
        ' name="Abbot" />\n' \
        '  <card count="1" expansion="Legacy of Blood" id="14"' \
        ' name="Abombwe" />\n ' \
        ' <card count="1" expansion="Promo-20051001" id="19"' \
        ' name="Alan Sovereign (Advanced)" />\n' \
        '</physicalcardset>'


class PhysicalCardSetWriterTests(SutekhTest):
    """class for the Physical Card Set writer tests"""

    def test_physical_card_set_writer(self):
        """Test physical card set writing"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        aAddedPhysCards = get_phys_cards()
        # We have a physical card list, so create some physical card sets
        oPhysCardSet1 = PhysicalCardSet(name=aCardSetNames[0])
        oPhysCardSet1.comment = 'A test comment'
        oPhysCardSet1.author = 'A test author'

        for iLoop in range(5):
            oPhysCardSet1.addPhysicalCard(aAddedPhysCards[iLoop].id)
            oPhysCardSet1.syncUpdate()

        # Check output

        oWriter = PhysicalCardSetWriter()
        sWriterXML = oWriter.gen_xml_string(oPhysCardSet1.name)
        self.assertEqual(sWriterXML,
            oWriter.gen_xml_string(aCardSetNames[0]))
        # The writer uses database order - this is not
        # the same across databases, hence the nature of the checks below
        self.assertEqual(sWriterXML, sExpected1)
        self.assertEqual(len(sWriterXML), len(sExpected1))

        sTempFileName =  self._create_tmp_file()
        fOut = open(sTempFileName, 'w')
        oWriter.write(fOut, aCardSetNames[0])
        fOut.close()

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        self.assertEqual(sData, sWriterXML)
        self.assertEqual(len(sData), len(sExpected1))
        self.assertEqual(sorted(sData), sorted(sExpected1))

        oPhysCardSet2 = PhysicalCardSet(name=aCardSetNames[1])
        oPhysCardSet2.author = 'A test author'
        oPhysCardSet2.comment = 'A formatted test comment\nA second line\n' \
                'A third line'
        oPhysCardSet2.annotations = 'Some Annotations'

        for iLoop in range(5, 10):
            oPhysCardSet2.addPhysicalCard(aAddedPhysCards[iLoop].id)
            oPhysCardSet2.syncUpdate()

        sWriterXML = oWriter.gen_xml_string(oPhysCardSet2.name)
        # The writer uses database order - this is not
        # the same across databases, hence the nature of the checks below
        self.assertEqual(len(sWriterXML), len(sExpected2))
        self.assertEqual(sorted(sWriterXML), sorted(sExpected2))


if __name__ == "__main__":
    unittest.main()



# SLDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Secret Library Deck Parser
# Copyright 2009 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Parser for Secret Library Web API deck format.

   For a description of the format see the Secret Library API at
   http://www.secretlibrary.info/api.php.

   The example deck given in the API documentation is:

   ***SL***TITLE***
   NAME OF THE DECK
   ***SL***AUTHOR***
   CREATOR OF THE DECK
   ***SL***CREATED***
   YYYY-MM-DD
   ***SL***DESCRIPTION***
   DESCRIPTION OF THE DECK
   MAY SPAN ON MULTIPLE LINES
   ***SL***CRYPT***
   2 Count Germaine
   2 Count Germaine (Adv)
   2 Fran\xc3\xa7ois Warden Loehr
   ***SL***LIBRARY***
   10 Cloak the Gathering
   2 Coven, The
   2 Carlton Van Wyk (Hunter)
   ***SL***ENDDECK***
   """

import re
from sutekh.core.SutekhObjects import csv_to_canonical

class SLDeckParser(object):
    """Parser for the Secret Library Web API deck format."""

    oCardLineRegexp = re.compile('^(?P<num>[0-9]+)\s+(?P<name>.*)$')

    def __init__(self, oHolder):
        self._oHolder = oHolder
        self._dSectionParsers = {
            'title': self._title_section,
            'author': self._author_section,
            'created': self._created_section,
            'description': self._description_section,
            'crypt': self._crypt_section,
            'library': self._library_section,
            'enddeck': self._no_section
        }
        self._fLineParser = self._no_section

    def reset(self):
        """Reset the parser state"""
        pass

    # section parsers

    def _switch_section(self, sLine):
        """Switch to a new section parser based on the given heading line."""
        sSection = sLine[len('***SL***'):-len('***')].lower()
        if sSection in self._dSectionParsers:
            self._fLineParser = self._dSectionParsers[sSection]
        else:
            raise RuntimeError('Unknown section heading in Secret'
                ' Library Deck Format')

    def _no_section(self, sLine):
        """Initial parser -- seeing a line here is an error."""
        raise RuntimeError('Data line outside of section'
            ' for Secret Library Deck format')

    def _title_section(self, sLine):
        """Parse a title line."""
        self._oHolder.name = sLine

    def _author_section(self, sLine):
        """Parse an author line."""
        self._oHolder.author = sLine

    def _created_section(self, sLine):
        """Parse a created section line."""
        sCreatedLine = "Created on %s" % (sLine,)
        if self._oHolder.comment:
            self._oHolder.comment += "\n" + sCreatedLine
        else:
            self._oHolder.comment = sCreatedLine

    def _description_section(self, sLine):
        """Parse a description line. """
        if self._oHolder.comment:
            self._oHolder.comment += "\n" + sLine
        else:
            self._oHolder.comment = sLine

    def _crypt_section(self, sLine):
        """Parse a crypt entry."""
        oMatch = self.oCardLineRegexp.match(sLine)
        if oMatch is None:
            raise RuntimeError('Unrecognised crypt line for Secrety Library'
                ' deck format')
        iNum = int(oMatch.group('num'))
        sName = oMatch.group('name')

        # Fix advanced
        if sName.endswith('(Adv)'):
            sName = sName.replace('(Adv)', '(Advanced)')

        # Avoid going down the exception path in IAbstractCard if we can
        sName = csv_to_canonical(sName)

        # Secret Library has no expansion info
        self._oHolder.add(iNum, sName, None)

    def _library_section(self, sLine):
        """Parse a library entry."""
        oMatch = self.oCardLineRegexp.match(sLine)
        if oMatch is None:
            raise RuntimeError('Unrecognised library line for Secrety Library'
                ' deck format')
        iNum = int(oMatch.group('num'))
        sName = oMatch.group('name')

        # Avoid going down the exception path in IAbstractCard if we can
        sName = csv_to_canonical(sName)

        # Secret Library has no expansion info
        self._oHolder.add(iNum, sName, None)

    def feed(self, sLine):
        """Feed the next line to the parser and extract the cards"""
        sLine = sLine.strip()
        if not sLine:
            # skip blank lines
            return

        if sLine.startswith('***'):
            self._switch_section(sLine)
        else:
            self._fLineParser(sLine)
